@@ -30,7 +30,6 @@
 #include <gconf/gconf-client.h>
 #include "gnome-vfs-i18n.h"
 #include "gnome-vfs-volume-monitor-private.h"
-#include "gnome-vfs-volume-monitor-daemon.h"
 #include "gnome-vfs-volume.h"
 #include "gnome-vfs-utils.h"
 #include "gnome-vfs-drive.h"
@@ -153,11 +152,13 @@ force_probe (void)
 	GnomeVFSClient *client;
 	GNOME_VFS_Daemon daemon;
 	CORBA_Environment  ev;
+	GnomeVFSDaemonForceProbeCallback callback;
 	
 	volume_monitor = gnome_vfs_get_volume_monitor ();
 
 	if (gnome_vfs_get_is_daemon ()) {
-		gnome_vfs_volume_monitor_daemon_force_probe (GNOME_VFS_VOLUME_MONITOR_DAEMON (volume_monitor));
+		callback = _gnome_vfs_get_daemon_force_probe_callback();
+		(*callback) (GNOME_VFS_VOLUME_MONITOR (volume_monitor));
 	} else {
 		client = _gnome_vfs_get_client ();
 		daemon = _gnome_vfs_client_get_daemon (client);
@@ -331,12 +332,12 @@ mount_unmount_operation (const char *mount_point,
 
 	
 #ifdef USE_VOLRMMOUNT
-       name = strrchr (mount_point, '/');
-       if (name != NULL) {
-               name = name + 1;
-       } else {
-	       name = mount_point;
-       }
+	name = strrchr (mount_point, '/');
+	if (name != NULL) {
+		name = name + 1;
+	} else {
+		name = mount_point;
+	}
 #else
        name = mount_point;
 #endif
@@ -482,7 +483,16 @@ unmount_connected_server (GnomeVFSVolume *volume,
 	g_free (detailed_error);
 }
 
-
+/** 
+ * gnome_vfs_volume_unmount:
+ * @volume:
+ * @callback:
+ * @user_data:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_volume_unmount (GnomeVFSVolume *volume,
 			  GnomeVFSVolumeOpCallback  callback,
@@ -491,7 +501,14 @@ gnome_vfs_volume_unmount (GnomeVFSVolume *volume,
 	char *mount_path, *device_path;
 	char *uri;
 	GnomeVFSVolumeType type;
-	
+
+	if (volume->priv->drive != NULL) {
+		if (volume->priv->drive->priv->must_eject_at_unmount) {
+			gnome_vfs_volume_eject (volume, callback, user_data);
+			return;
+		}
+	}
+
 	emit_pre_unmount (volume);
 
 	type = gnome_vfs_volume_get_volume_type (volume);
@@ -512,6 +529,16 @@ gnome_vfs_volume_unmount (GnomeVFSVolume *volume,
 	}
 }
 
+/** 
+ * gnome_vfs_volume_eject:
+ * @volume:
+ * @callback:
+ * @user_data:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_volume_eject (GnomeVFSVolume *volume,
 			GnomeVFSVolumeOpCallback  callback,
@@ -541,6 +568,16 @@ gnome_vfs_volume_eject (GnomeVFSVolume *volume,
 	}
 }
 
+/** 
+ * gnome_vfs_drive_mount:
+ * @drive:
+ * @callback:
+ * @user_data:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_drive_mount (GnomeVFSDrive  *drive,
 		       GnomeVFSVolumeOpCallback  callback,
@@ -561,65 +598,96 @@ gnome_vfs_drive_mount (GnomeVFSDrive  *drive,
 	g_free (device_path);
 }
 
+/** 
+ * gnome_vfs_drive_unmount:
+ * @drive:
+ * @callback:
+ * @user_data:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_drive_unmount (GnomeVFSDrive  *drive,
 			 GnomeVFSVolumeOpCallback  callback,
 			 gpointer                   user_data)
 {
-	char *mount_path, *device_path, *uri;
-	GnomeVFSVolume *volume;
+	GList *vol_list;
+	GList *current_vol;
 
-
-	volume = gnome_vfs_drive_get_mounted_volume (drive);
-
-	if (volume != NULL) {
-		emit_pre_unmount (volume);
+	if (drive->priv->must_eject_at_unmount) {
+		gnome_vfs_drive_eject (drive, callback, user_data);
+		return;
 	}
 
-	uri = gnome_vfs_drive_get_activation_uri (drive);
-	mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	device_path = gnome_vfs_drive_get_device_path (drive);
-	mount_unmount_operation (mount_path,
-				 device_path,
-				 (volume == NULL) ? GNOME_VFS_DEVICE_TYPE_UNKNOWN:gnome_vfs_volume_get_device_type (volume),
-				 FALSE, FALSE,
-				 callback, user_data);
-	g_free (mount_path);
-	g_free (device_path);
-	gnome_vfs_volume_unref (volume);
+	vol_list = gnome_vfs_drive_get_mounted_volumes (drive);
+
+	for (current_vol = vol_list; current_vol != NULL; current_vol = current_vol->next) {
+		GnomeVFSVolume *vol;
+		vol = GNOME_VFS_VOLUME (current_vol->data);
+
+		gnome_vfs_volume_unmount (vol,
+					  callback,
+					  user_data);
+	}
+
+	gnome_vfs_drive_volume_list_free (vol_list);
 }
 
+/** 
+ * gnome_vfs_drive_eject:
+ * @drive:
+ * @callback:
+ * @user_data:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_drive_eject (GnomeVFSDrive  *drive,
 		       GnomeVFSVolumeOpCallback  callback,
 		       gpointer                   user_data)
 {
-	char *mount_path, *device_path, *uri;
-	GnomeVFSVolume *volume;
+	GList *vol_list;
+	GList * current_vol;
 
+	vol_list = gnome_vfs_drive_get_mounted_volumes (drive);
 
-	volume = gnome_vfs_drive_get_mounted_volume (drive);
+	for (current_vol = vol_list; current_vol != NULL; current_vol = current_vol->next) {
+		GnomeVFSVolume *vol;
+		vol = GNOME_VFS_VOLUME (current_vol->data);
+
+		/* Check to see if this is the last volume */
+		/* If not simply unmount it */
+		/* If so the eject the media along with the unmount */
+		if (current_vol != NULL) { 
+			gnome_vfs_volume_unmount (vol,
+						  callback,
+						  user_data);
+		} else { 
+			gnome_vfs_volume_eject (vol,
+						callback,
+						user_data);
+		}
 
-	if (volume != NULL) {
-		emit_pre_unmount (volume);
 	}
 
-	uri = gnome_vfs_drive_get_activation_uri (drive);
-	mount_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	device_path = gnome_vfs_drive_get_device_path (drive);
-	mount_unmount_operation (mount_path,
-				 device_path,
-				 (volume == NULL) ? GNOME_VFS_DEVICE_TYPE_UNKNOWN:gnome_vfs_volume_get_device_type (volume),
-				 FALSE, TRUE,
-				 callback, user_data);
-	g_free (mount_path);
-	g_free (device_path);
-	gnome_vfs_volume_unref (volume);
+	gnome_vfs_drive_volume_list_free (vol_list);
 }
 
 
+/** 
+ * gnome_vfs_connect_to_server:
+ * @uri:
+ * @display_name:
+ * @icon:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_connect_to_server (char                     *uri,
 			     char                     *display_name,
