@@ -161,7 +161,6 @@ static gboolean sftp_connection_process_errors (GIOChannel *channel,
 						GIOCondition cond,
 						GnomeVFSResult *status);
 
-
 
 static guint inited_buffers = 0;
 
@@ -647,8 +646,6 @@ buffer_write_file_info (Buffer *buf, const GnomeVFSFileInfo *info, GnomeVFSSetFi
 	}
 }
 
-
-
 static GnomeVFSResult 
 sftp_status_to_vfs_result (guint status) 
 {
@@ -676,7 +673,6 @@ sftp_status_to_vfs_result (guint status)
 	}
 }
 
-
 
 /* Derived from OpenSSH, sftp-client.c:get_status */
 
@@ -844,8 +840,6 @@ iobuf_send_string_request_with_file_info (int                      fd,
 	buffer_free (&msg);
 }
 
-
-
 static char*
 get_user_from_uri_or_password_line (const GnomeVFSURI *uri,
 				    const char *password_line)
@@ -904,8 +898,6 @@ get_authtype_from_password_line (const char *password_line)
 	return g_strdup ("password");
 }
 
-
-
 static gboolean
 invoke_fill_auth (const GnomeVFSURI *uri,
 		  const char *password_line,
@@ -1702,8 +1694,6 @@ sftp_connection_unref (SftpConnection *conn)
 	}
 }
 
-
-
 /* Portions of the below functions inspired by functions in OpenSSH sftp-client.c */
 
 static GnomeVFSResult
@@ -1769,8 +1759,6 @@ get_real_path (SftpConnection *conn, const gchar *path, gchar **realpath)
 	return GNOME_VFS_OK;
 }
 
-
-
 static GnomeVFSResult 
 do_open (GnomeVFSMethod        *method,
 	 GnomeVFSMethodHandle **method_handle,
@@ -1809,7 +1797,8 @@ do_open (GnomeVFSMethod        *method,
 	sftp_mode = 0;
 	if (mode & GNOME_VFS_OPEN_READ) sftp_mode |= SSH2_FXF_READ;
 	if (mode & GNOME_VFS_OPEN_WRITE) sftp_mode |= SSH2_FXF_WRITE;
-	if ((mode & GNOME_VFS_OPEN_WRITE) && !(mode & GNOME_VFS_OPEN_RANDOM))
+	if ((mode & GNOME_VFS_OPEN_TRUNCATE) ||
+	    ((mode & GNOME_VFS_OPEN_WRITE) && !(mode & GNOME_VFS_OPEN_RANDOM)))
 		sftp_mode |= SSH2_FXF_TRUNC;
 
 	buffer_write_gint32 (&msg, sftp_mode);
@@ -1991,6 +1980,8 @@ do_read (GnomeVFSMethod       *method,
 	guchar *curr_ptr;
 	gboolean out_of_order;
 	GnomeVFSResult result;
+	gboolean got_eof;
+	guint outstanding;
 
 	struct ReadRequest 
 	{
@@ -2003,6 +1994,8 @@ do_read (GnomeVFSMethod       *method,
 
 	DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Enter", __FUNCTION__));
 
+	got_eof = FALSE;
+	
 	handle = SFTP_OPEN_HANDLE (method_handle);
 	*bytes_read = 0;
 	num_req = 0;
@@ -2010,6 +2003,7 @@ do_read (GnomeVFSMethod       *method,
 	req_svc_ptr = 0;
 	curr_ptr = buffer_in;
 	buffer = buffer_in;
+	outstanding = 0;
 
 	read_req = g_new0 (struct ReadRequest, max_req);
 
@@ -2017,7 +2011,8 @@ do_read (GnomeVFSMethod       *method,
 
 	sftp_connection_lock (handle->connection);
 
-	while (*bytes_read < num_bytes) {
+	while ((*bytes_read < num_bytes) ||
+	       (outstanding > 0)) {
 		while (num_req < max_req && curr_ptr < buffer + num_bytes) {
 			read_req[req_ptr].id = sftp_connection_get_id (handle->connection);
 			read_req[req_ptr].req_len =
@@ -2029,6 +2024,7 @@ do_read (GnomeVFSMethod       *method,
 				      __FUNCTION__, req_ptr, read_req[req_ptr].id,
 				      read_req[req_ptr].req_len, read_req[req_ptr].ptr));
 
+			outstanding++;
 			iobuf_send_read_request (handle->connection->out_fd,
 						 read_req[req_ptr].id,
 						 handle->offset + (read_req[req_ptr].ptr - buffer),
@@ -2044,6 +2040,7 @@ do_read (GnomeVFSMethod       *method,
 
 		buffer_clear (&msg);
 		result = buffer_recv (&msg, handle->connection->in_fd);
+		outstanding--;
 
 		if (result != GNOME_VFS_OK) {
 			sftp_connection_unlock (handle->connection);
@@ -2082,15 +2079,20 @@ do_read (GnomeVFSMethod       *method,
 			DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: req_svc = %d, req_ptr = %d",
 				      __FUNCTION__, req_svc, req_ptr));
 
-			if (status != SSH2_FX_EOF || read_req[req_svc].ptr == buffer) {
+			if (status != SSH2_FX_EOF) {
+				DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: status return %d",
+					      __FUNCTION__, sftp_status_to_vfs_result (status)));
 				buffer_free (&msg);
 				g_free (read_req);
 				sftp_connection_unlock (handle->connection);
 				return sftp_status_to_vfs_result (status);
 			}
 
+			if (read_req[req_svc].ptr == buffer)
+				got_eof = TRUE;
+			
 			/* We hit an EOF, so make sure we don't try any more reads after this */
-			num_bytes = read_req[req_svc].ptr - buffer;
+			num_bytes = MIN (num_bytes, read_req[req_svc].ptr - buffer);
 
 			/* Kill this request */
 			read_req[req_svc].id = 0;
@@ -2109,6 +2111,7 @@ do_read (GnomeVFSMethod       *method,
 				read_req[req_svc].req_len -= len;
 				read_req[req_svc].ptr += len;
 
+				outstanding++;
 				iobuf_send_read_request
 					(handle->connection->out_fd,
 					 read_req[req_svc].id,
@@ -2140,6 +2143,9 @@ do_read (GnomeVFSMethod       *method,
 
 	sftp_connection_unlock (handle->connection);
 
+	if (got_eof)
+		return GNOME_VFS_ERROR_EOF;
+	
 	return GNOME_VFS_OK;
 }
 
@@ -2688,7 +2694,6 @@ do_make_directory (GnomeVFSMethod  *method,
 	return res;
 }
 
-
 static GnomeVFSResult
 do_remove_directory (GnomeVFSMethod  *method,
 		     GnomeVFSURI     *uri,
@@ -2742,6 +2747,18 @@ do_move (GnomeVFSMethod  *method,
 
 	id = sftp_connection_get_id (conn);
 
+	/* if force_replace is specified, try to remove new_uri */
+	if (force_replace) {
+		iobuf_send_string_request (conn->out_fd,
+					   id,
+					   SSH2_FXP_REMOVE,
+					   new_path,
+					   strlen (new_path));
+		res = iobuf_read_result (conn->in_fd, id);
+		if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_NOT_FOUND)
+			goto bail;
+	}
+
 	buffer_write_gchar (&msg, SSH2_FXP_RENAME);
 	buffer_write_gint32 (&msg, id);
 	buffer_write_string (&msg, old_path);
@@ -2749,11 +2766,12 @@ do_move (GnomeVFSMethod  *method,
 	buffer_send (&msg, conn->out_fd);
 	buffer_free (&msg);
 
+	res = iobuf_read_result (conn->in_fd, id);
+
+ bail:
 	g_free (old_path);
 	g_free (new_path);
 
-	res = iobuf_read_result (conn->in_fd, id);
-
 	sftp_connection_unref (conn);
 	sftp_connection_unlock (conn);
 
@@ -2953,7 +2971,6 @@ do_create_symlink (GnomeVFSMethod   *method,
 	return res;
 }
 
-
 
 static GnomeVFSMethod method = {
 	sizeof (GnomeVFSMethod),
